# -----------------------------------------------------------------------------
package admin_add_a_user;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 Feb 20: 
#          
# -- 
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

use admin_bits;

use config;
use debug;
use exceptions;

use Passwd;

# -----------------------------------------------------------------------------

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(print_add_user_form
             actually_add_user);

# -----------------------------------------------------------------------------
# -- MODULE/PACKAGE VARS :
# -----------------------------------------------------------------------------

my $passwd_dir   = &passwd_dir;
my $html_lib_dir = &html_lib_dir;

# -----------------------------------------------------------------------------
# -- EXPORTS :

sub print_add_user_form() {

  my $USERNAME = shift;
  my $TREE     = shift;

    my $passwd       = Passwd->new;
    my $groups       = $passwd->{GROUPS};
    my $passwd_files = $passwd->{PASSWDFILES};

    my $textline;

    open (FORM, "$html_lib_dir/admin_add_user_form.html_src") 
        || &raise_stop_exception("Can't open admin_add_user_form.html_src");

    while ($textline = <FORM>) {
        $textline =~ s/%tree%/$TREE/;
        $textline =~ s/%groups%/@$groups/;
        $textline =~ s/%passwdfiles%/@$passwd_files/;
        $textline =~ s/%username%/$USERNAME/;
        print $textline;
      }

    close (FORM);
  }

# -----------------------------------------------------------------------------

sub actually_add_user() {

  my $newusername = shift;
  my $password    = shift;
  my $realname    = shift;
  my $passwd_file = shift;
  my $groups      = shift;
  my $username    = shift;
  my $tree        = shift;

    my ($p_crypt, $salt);

    # -- does the username already exist, do the groups already exist,
    #    does the password file exist, is the password sensible?

    my $passwd = Passwd->new;

    unless ($passwd->ok_username($newusername)) {
        &raise_info_exception("Username: $newusername, not OK.");
      }

    unless ($passwd->ok_groups($groups)) {
        &raise_info_exception("Groups: <TT>$groups</TT>, not OK.");
      }

    unless ($passwd->ok_passwd_file($passwd_file)) {
        &raise_info_exception("Password file not OK.");
      }

    unless ($passwd->ok_password($password)) {
        &raise_info_exception("Password not OK.");
      }


    my $title_refresh = &refresh_to_admin_menu_alt_too($tree, $username,
                         "Gonna add user:");

    print $title_refresh;

    print "<TABLE>" .
          "<TR><TD><B>Username:     </B></TD>" . 
          "<TD><TT>$newusername</TT></TD></TR>" .
          "<TR><TD><B>Password:     </B></TD>" .
          "<TD><TT>$password</TT></TD></TR>" .
          "<TR><TD><B>Realname:     </B></TD>" . 
          "<TD><TT>$realname</TT></TD></TR>" .
          "<TR><TD><B>Password File:</B></TD>" . 
          "<TD><TT>$passwd_file</TT></TD></TR>" .
          "<TR><TD><B>Groups:       </B></TD>" . 
          "<TD><TT>$groups</TT></TD></TR></TABLE>\n";

    $salt = crypt($password, "pepper");
    $p_crypt = crypt($password, $salt);

    if (-e "$passwd_dir/$passwd_file") {
        open(ADDU, ">>$passwd_dir/$passwd_file")
            || &raise_stop_exception("Unable to append entry to " . 
                                      "password file.");
      }
    else {
        open(ADDU, ">$passwd_dir/$passwd_file")
            || &raise_stop_exception("Unable to create password file: " . 
                                     "$passwd_dir/$passwd_file.");
      }
    print ADDU "\n $newusername & $p_crypt & $realname & $groups";
    close(ADDU);
  }


# -----------------------------------------------------------------------------
# -- PRIVATES :



# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
