# -----------------------------------------------------------------------------
package admin_tool;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 Feb 20: 
#          
# -- 
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

use config;
use debug;
use exceptions;

use Passwd;

# -----------------------------------------------------------------------------

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(print_login_form
             print_admin_menu
             valid_admin_uname_passwd
             review_passwd_dir
             review_indiv_user_access_control);

# -----------------------------------------------------------------------------
# -- MODULE/PACKAGE VARS :
# -----------------------------------------------------------------------------

my $html_lib_dir = &html_lib_dir;
my $passwd_admin_dir = &passwd_admin_dir;

# -----------------------------------------------------------------------------
# -- EXPORTS :


sub print_login_form() {

    my $textline;
    my $tree_selector = "<B>Available Trees: </B>";
    my @a_trees       = &available_trees;

    $tree_selector .= "<SELECT NAME=\"TREE\" SIZE=1>\n";
    foreach (@a_trees) {$tree_selector .= "<OPTION>$_\n";}
    $tree_selector .= "</SELECT>\n"; 

    open (FORM, "$html_lib_dir/admin_login_form.html_src") 
        || &raise_stop_exception("Can't open admin_login_form.html_src");

    while ($textline = <FORM>) {
        $textline =~ s/%treeselector%/$tree_selector/;
        print $textline;
      } 

    close(FORM);
  }

# -----------------------------------------------------------------------------
# -- wander through all admin password files to see if given uname and 
#    password are valid :

sub valid_admin_uname_passwd() {

  my $username = shift @_;
  my $password = shift @_;
  
    my ($filename, $textline, $u, $p, $r, $salt, $password_crypt);

    &debug_h("valid_admin_uname_password");

    # -- look in our password-file-directory for password files;  look in
    #    each in turn :

    opendir(PASSWDDIR, $passwd_admin_dir) 
        || &raise_logout_exception("passwd_dir_unavail"); 

    foreach $filename (sort readdir(PASSWDDIR)) {

        if ($filename eq ".") {next};
        if ($filename eq "..") {next};
        if ($filename =~ m/[^~]*~/) {next};

        &debug_i("reading password file: $filename");

        # -- open our password file;  read in username/password combinations 
        #    from our password file :

        open(PASSWDFILE, "$passwd_admin_dir/$filename") 
            || &raise_logout_exception('passwd_file_unavail'); 
        
        while ($textline = <PASSWDFILE>) {

            &debug_i("password file entry: $textline");

            ($u, $p, $r) = split(/&/, $textline);  
            $u =~ s/\s*//g;  
            $p =~ s/\s*//g;            

            &debug_i("Username: $u"); 
            &debug_i("Password: $p"); 

            if ($username eq $u) {
                $salt = crypt($password, "pepper");
                $password_crypt = crypt($password, $salt);
                if ($password_crypt eq $p) {return 1;}
              } 

          } # ...while looking through password file;
        close(PASSWDFILE);

      } # ...foreach password file in password directory;
    closedir(PASSWDDIR);

    # -- if got to here they ain't valid :
    return 0;
  }

# -----------------------------------------------------------------------------

sub print_admin_menu() {

  my $TREE     = shift;
  my $USERNAME = shift;
  
    print "\n\n <H2>Admin Tool Menu ( Tree: $TREE, U: $USERNAME)</H2>\n\n";

    print "\n<TABLE CELLSPACING=10>";

    print "\n<TR><TD VALIGN=\"top\"><B>Groups</B></TD><TD>";
    print "\n<UL>";
    print "\n<LI><A HREF=\"admin_groups.pl?TREE=$TREE&USERNAME=$USERNAME\">" . 
          "Add a group</A>";
    print "\n<LI><A HREF=\"admin_groups.pl?TREE=$TREE&USERNAME=$USERNAME&" . 
          "AGACTION=selectremovegroupsusers\">" . 
          "Remove Users from a Group</A>";
    print "\n<LI><A HREF=\"admin_groups.pl?TREE=$TREE&USERNAME=$USERNAME&" . 
          "AGACTION=selectaddgroupsusers\">" . 
          "Add Users to a Group</A>";
    print "\n</UL> ";
    print "\n</TD></TR>";

    print "\n<TR><TD VALIGN=\"top\"><B>Permissions</B></TD><TD>";
    print "\n<UL> ";
    print "\n<LI><A HREF=\"admin_tool.pl?" . 
           "TREE=$TREE&USERNAME=$USERNAME&" .
           "ACTION=REVIEW_PASSWD_DIR\">" . 
           "Review password files, groups and users</A>"; 

    print "\n<LI><A HREF=\"admin_change_permissions_cdf.pl?" . 
          "TREE=$TREE&USERNAME=$USERNAME&SELECTCDF=yes\">" . 
          "Change permissions</A> for a branch"; 
    print "\n</UL>";
    print "\n</TD></TR>";
    

    print "\n<TR><TD VALIGN=\"top\"><B>Users</B></TD><TD>";
    print "\n<UL> ";
    print "\n<LI><A HREF=\"admin_add_a_user.pl?" . 
          "TREE=$TREE&USERNAME=$USERNAME\">Add a user</A>.";
    print "\n</UL>";
    print "\n</TD></TR>";

    print "\n<TR><TD VALIGN=\"top\"><B>Branches</B></TD><TD>";
    print "\n<UL> ";
    print "\n<LI>Add or delete " . 
          "<A HREF=\"admin_add_a_branch.pl?" . 
          "TREE=$TREE&USERNAME=$USERNAME&SELECTLOCAT=yes\">a branch</A>";
    print "\n</UL>";
    print "\n</TD></TR>";

    print "\n<TR><TD VALIGN=\"top\"><B>Trees</B></TD><TD>";
    print "\n<UL>";
    print "\n<LI><A HREF=\"admin_view_tree.pl?" . 
          "TREE=$TREE&BY=structure\">View</A> tree: " .
          "structure, ACLs, READMEs.";
    print "\n<LI><A HREF=\"admin_add_a_tree.pl\">Add</A> a tree.";
    print "\n<LI> Check integrity of the current tree.";
    print "\n</UL>";
    print "\n</TD></TR>";

    print "\n<TR><TD VALIGN=\"top\"><B>Entries</B></TD><TD>";
    print "\n<UL>";
    print "\n<LI> View entries by: " . 
          "<A HREF=\"admin_view_entries.pl?" . 
          "TREE=$TREE&USERNAME=$USERNAME". 
          "\&BY=structure\">structure</A>, " . 
          "<A HREF=\"admin_view_entries.pl?" . 
          "TREE=$TREE&USERNAME=$USERNAME". 
          "\&BY=revcron\">rev-chron</A> " . 
          "(and delete them).";
    print "\n</UL> ";
    print "\n</TD></TR>";

    print "\n</TABLE>";
  }


# -----------------------------------------------------------------------------
# -- 

sub review_passwd_dir {
  my $query    = shift;
  my $TREE     = shift;
  my $USERNAME = shift;

    my $passwd = Passwd->new();
   
    print "\n<H1>Current Status of Password Directory</H1>";
    print "\n<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=5>";

    my $pwfile;

    foreach $pwfile (keys %{$passwd->{PASSWD}}) {
        print "\n<TR><TD COLSPAN=3><BIG><B>$pwfile</B></BIG></TD></TR>\n";

        my $uname;
            
        foreach $uname (keys %{$passwd->{PASSWD}->{$pwfile}}) {
            print "\n<TR><TD><FONT COLOR=\"white\">aaaaaa</TD>" . 
                  "<TD><B>$uname</B></TD><TD></TD><TD>" . 
                  "$passwd->{PASSWD}->{$pwfile}->{$uname}->{RNAME}" . 
                  "</TD><TD></TD><TD>";
            foreach (@{$passwd->{PASSWD}->{$pwfile}->{$uname}->{GRPS}}) {
                print " $_, ";
              }
            print "\n</TD></TR>\n";
          }
      }
    print "\n</TABLE>";
  }


# -----------------------------------------------------------------------------
# -- PRIVATES :





# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------

