# -----------------------------------------------------------------------------
package Cookie;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 Oct 30 written from cookies.pm;
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

use config;
use Debug;


# -----------------------------------------------------------------------------
# -- CONFIG/INIT :

my $cookie_dir       = &cookie_dir;
my $admin_cookie_dir = &admin_cookie_dir;


# -----------------------------------------------------------------------------
# -- CONSTRUCTOR :
# -----------------------------------------------------------------------------

sub new() {

    my $class = shift;
    my $doc   = shift;
    my $self  = { };

    bless($self, $class);

    $self->{COOKIES} = &read_env_cookies($doc);

    return $self;
  }


# -----------------------------------------------------------------------------
# -- CONSTRUCTOR SLAVES :
# -----------------------------------------------------------------------------

sub read_env_cookies() {
    
    my $doc     = shift;
    my @cookies = split(/;/, $ENV{'HTTP_COOKIE'});

    my $debug = Debug->new(9, "Gonna loop through cookies list");

    foreach (@cookies) {
        $debug->item("cookie: $_");
      }

    $debug->wen($doc);
    return \@cookies;
  }


# -----------------------------------------------------------------------------
# -- PUBLIC METHODS :
# -----------------------------------------------------------------------------


sub new_cookie() {
    
  my $self     = shift;
  my $USERNAME = shift;

    # --- fields for a cookie :
    my $key = "simonh_csu_umist_doc_lib";
    my $value = rand();

    # --- store this cookie value for other scripts to use:
    if (-e "/var/local/doc_U_base/fpigs_cookie_jar") {
        open(CJAR, ">>/var/local/doc_U_base/fpigs_cookie_jar");
      }
    else {
        open(CJAR, ">/var/local/doc_U_base/fpigs_cookie_jar");
      }
    print CJAR "$value $USERNAME " . time ." \n";
    close(CJAR);

    return ($key, $value);
  }

# -----------------------------------------------------------------------------

sub new_admin_cookie() {
    
  my $self = shift;

    # --- fields for a cookie :
    my $key = "simonh_csu_umist_doc_lib_admin";
    my $value = rand();

    # --- store this cookie value for other scripts to use:
    if (-e "/var/local/doc_U_base_admin/fpigs_cookie_jar") {
        open(CJAR, ">>/var/local/doc_U_base_admin/fpigs_admin_cookie_jar");
      }
    else {
        open(CJAR, ">/var/local/doc_U_base_admin/fpigs_admin_cookie_jar");
      }
    print CJAR $value, "\n";
    close(CJAR);

    return ($key, $value);
  }

# -----------------------------------------------------------------------------

sub valid_cookie() {

  my $self     = shift;
  my $doc      = shift;
  my $except   = shift;
  my $USERNAME = shift;

    my ($cookie_file, $key, $value);
    my ($candidate, $textline);

    my $debug = Debug->new(9, "Gonna check if given cookie is valid");

    unless (opendir(COOKIEDIR, $cookie_dir)) {
        $except->raise_logout("cookie_dir_not_opened: $cookie_dir");
      }

    # -- optimise order of cookie-jar-files looked at?                   ##!!##
    while ($cookie_file = readdir(COOKIEDIR)) {

        unless ($cookie_file =~ m/^fpigs_cookie_jar/) {next;}
 

        foreach $candidate (@{$self->{COOKIES}}) {
            ($key, $value) = split(/=/, $candidate);

            $debug->item("Candidate cookie: $candidate");

            open(COOKIEFILE, $cookie_dir . $cookie_file)
                || $except->raise_logout("cookie_file_not_opened");
       
            $debug->item("cookie file: $cookie_file");
            while($textline = <COOKIEFILE>){
                my @fields = split(/\s/, $textline);

######print "\n fields 0,1 " . $fields[0] . " " . $fields[1];

                if ($fields[0] eq $value) {
######print "\n FOUND";
                    $debug->item("FOUND matching cookie..." . 
                                 "($textline, $value) ");
                    closedir(COOKIEDIR);
#######print "\n ......" . $USERNAME . " " . $fields[1];
                    if ($fields[1] eq $USERNAME) {
                        $debug->wen($doc);
                        return 1;
                      }
                    else {
                      $debug->wen($doc);
                      return 0;
                    }
                  }
                else {
                    $debug->item("Still looking for matching cookie..." . 
                                  "$textline, $value");
                  }
              }
          }
      }

    closedir(COOKIEDIR);
    $debug->wen($doc);
    return 0;
  }

# -----------------------------------------------------------------------------

sub valid_admin_cookie() {

  my $self   = shift;
  my $doc    = shift;
  my $except = shift;

    my ($cookie_file, $key, $value);
    my ($candidate, $textline);

    my $debug = Debug->new(9, "Gonna check if given _admin_ cookie is valid");

    unless (opendir(COOKIEDIR, $admin_cookie_dir)) {
        $except->raise_logout($self, $doc, 
                              "admin_cookie_dir_not_opened: " . 
                              "$admin_cookie_dir");
      }

    # -- optimise order of cookie-jar-files looked at?                   ##!!##
    while ($cookie_file = readdir(COOKIEDIR)) {

        unless ($cookie_file =~ m/^fpigs_admin_cookie_jar/) {next;}

        foreach $candidate (@{$self->{COOKIES}}) {
            ($key, $value) = split(/=/, $candidate);

            $debug->item("Candidate cookie: $candidate");

            open(COOKIEFILE, $admin_cookie_dir . $cookie_file)
                || $except->raise_logout("cookie_file_not_opened");
       
            $debug->item("cookie file: $cookie_file");

            while($textline = <COOKIEFILE>){
                chop($textline);

                if ($textline eq $value) {
                    $debug->item("FOUND matching cookie... ");
                    closedir(COOKIEDIR);
                    return 1;
                  }
                else {
                    $debug->item("Still looking for matching cookie..." . 
                                  "$textline, $value");
                  }
              }
          }
      }

    closedir(COOKIEDIR);
    $debug->wen();
    return 0;
  }

# -----------------------------------------------------------------------------

sub delete_jar_cookie() {
    
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------

