# -----------------------------------------------------------------------------
package Exception;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 Oct 30 written from exceptions.pm; 
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

# -----------------------------------------------------------------------------

use config;
use log;

# -----------------------------------------------------------------------------

my $errors_html_lib_dir = &errors_html_lib_dir;

# -----------------------------------------------------------------------------
# DOCUMENTATION :
# 
# raise_logout    :
#
#  -- that's it, it's all gone horribly wrong, so tidy up as best we can and
#     give up (force logout/exit/quit);
#
# raise_integrity :
#
#  -- something has gone wrong, so say so in no uncertain terms --- raised
#     when halfway through something so database integrity is lost;
#
# raise_stop      :
#
#  -- something has gone wrong, so say so in no uncertain terms and simply
#     stop;
# 
# raise_options   : 
#
#  -- something has gone wrong, so say so in no uncertain terms, but don't 
#     kill everything, rather offer some options;
#
# raise_info      : 
#
#  -- something has gone wrong, but not very seriously --- just put up 
#     some info;


# -----------------------------------------------------------------------------


sub new() {

    my $class = shift;
    my $self  = { };

    bless($self, $class);

    $self->{VSKIPABOVE} = "<BR><BR>";
    $self->{VSKIPBELOW} = "<BR><BR>";

    $self->{FONT_} = "<FONT SIZE=4><B>";
    $self->{_FONT} = "</B></FONT>";

    $self->{FONT_B_} = "<FONT SIZE=3><B>";
    $self->{_FONT_B} = "</B></FONT>";

    $self->{FONT_TT_} = "<FONT SIZE=3><TT>";
    $self->{_FONT_TT} = "</TT></FONT>";

    return $self;
  }


# -----------------------------------------------------------------------------
# -- this one is for serious shit, like someone trying to access things 
#    without a cookie (i.e., without authentication) :

# -- version 3 includes the call-stack, line numbers, etc. :

sub raise_logout_3() {
  my $self         = shift;
  my $cookies      = shift;
  my $doc          = shift;
  my $s            = shift;
  my $string       = shift;

    my $cs         = 0;
    my $call_stack = "";

    my ($package, $file, $line, $subname);

    while (($package, $file, $line, $subname, undef, undef) = caller($cs++)) {
        $call_stack .= "<BR>" 
                    . "Package: $package, File: $file, " 
     	            .  "Line: $line, Sub: $subname ";
      }  

    $doc->append($self->{VSKIPABOVE});
    $doc->append($self->{FONT_});
    $doc->append("Exception raised: \"$string\"");
    $doc->append($self->{_FONT});
    $doc->append("<BR>");
    $doc->append($self->{FONT_B_});
    $doc->append("<BR>");
    $doc->append("Call stack:");
    $doc->append($self->{_FONT_B});
    $doc->append("<BR>");
    $doc->append($self->{FONT_TT_});
    $doc->append($call_stack);
    $doc->append($self->{_FONT_TT});
    $doc->append($self->{VSKIPBELOW});

    # -- delete any trace of them :
    $cookies->delete_jar_cookie();                 
                                                   
    my @caller = caller(1);
    &log_exception_2($caller[1] . "::" . $caller[3],
                     $s->attr("USERNAME"), $s->attr("TREE"), 
                     $s->attr("LOCATION"), $string);

    $doc->wen();
    exit; 
  }


sub raise_logout_2() {
  my $self         = shift;
  my $cookies      = shift;
  my $doc          = shift;
  my $s            = shift;

  my $string       = shift;

    my $textline;

    $doc->append($self->{VSKIPABOVE});
    $doc->append($self->{FONT_});
    $doc->append($string);
    $doc->append($self->{_FONT});
    $doc->append($self->{VSKIPBELOW});

    # -- delete any trace of them :
    $cookies->delete_jar_cookie();                 
                                                   
    my @caller = caller(1);
    &log_exception_2($caller[1] . "::" . $caller[3],
                     $s->attr("USERNAME"), $s->attr("TREE"), 
                     $s->attr("LOCATION"), $string);

    $doc->wen();
    exit; 
  }


sub raise_logout() {

  my $self         = shift;
  my $cookies      = shift;
  my $doc          = shift;
  my $string       = shift;
  my $cookie_value = shift;

    my $textline;

    if ($string eq "invalid_cookie") {

        $doc->append($self->{VSKIPABOVE});
        $doc->append($self->{FONT_});
        $doc->append("You have no authorisation!  " . 
                     "Perhaps your cookie has expired?");
        $doc->append($self->{_FONT});
        $doc->append($self->{VSKIPBELOW});
      }
    elsif ($string eq "invalid_admin_cookie") {

        $doc->append($self->{VSKIPABOVE});
        $doc->append($self->{FONT_});
        $doc->append("You have no authorisation!  Are you Admin? " . 
                     "(Perhaps your cookie has expired?)");
        $doc->append($self->{_FONT});
        $doc->append($self->{VSKIPBELOW});
      }
    elsif ($string eq "dodgy_uname_passwd") {

        $doc->append($self->{VSKIPABOVE});
        $doc->append($self->{FONT_});
        $doc->append("Invalid username/password!");
        $doc->append($self->{_FONT});
        $doc->append($self->{VSKIPBELOW});
      }
    else {
        $doc->append("<H2><FONT COLOR=\"red\">$string</FONT></H2>\n");
      }

    # -- delete any trace of them :
    $cookies->delete_jar_cookie();                 
                                                   
    my @caller = caller(1);
    &log_exception($caller[0] . "::" . $caller[3] . " :  $string");

    $doc->wen();
    exit; 
  }


# -----------------------------------------------------------------------------
# -- this one is supposed to be used when the database is apparently 
#    upset :

sub raise_integrity() {

  my $self   = shift @_;
  my $doc    = shift;
  my $string = shift @_;

    $doc->append($self->{VSKIPABOVE});
    $doc->append($self->{FONT_});
    $doc->append($string);
    $doc->append($self->{_FONT});
    $doc->append($self->{VSKIPBELOW});

    my @caller = caller(1);
    &log_exception($caller[0] . "::" . $caller[3] . " :  $string");

    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------

sub raise_stop() {

  my $self   = shift @_;
  my $doc    = shift;
  my $string = shift @_;

    $doc->append($self->{VSKIPABOVE});
    $doc->append($self->{FONT_});
    $doc->append($string);
    $doc->append($self->{_FONT});
    $doc->append($self->{VSKIPBELOW});

    my @caller = caller(1);
    &log_exception($caller[0] . "::" . $caller[3] . " :  $string");

    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- stop processing, but leave user authenticated (cookied) :

sub raise_options() {

  my $self   = shift @_;
  my $doc    = shift;
  my $s      = shift;
  my $string = shift @_;

    $doc->append($self->{VSKIPABOVE});
    $doc->append($self->{FONT_});
    $doc->append($string);
    $doc->append($self->{_FONT});
    $doc->append($self->{VSKIPBELOW});

    my @caller = caller(1);
    &log_exception_2($caller[1] . "::" . $caller[3],
                     $s->attr("USERNAME"), $s->attr("TREE"), 
                     $s->attr("LOCATION"), $string);

    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- print out some info but let everything carry on apart from that:

sub raise_info() {

  my $self   = shift @_;
  my $string = shift @_;

    print $self->{VSKIPABOVE};
    print $self->{FONT_};
    print $string;
    print $self->{_FONT};
    print $self->{VSKIPBELOW};

    my @caller = caller(1);
    &log_exception($caller[0] . "::" . $caller[3] . " :  $string");
  }



# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
