# -----------------------------------------------------------------------------
package view_entries;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 March 01: 
#          
# -- 
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

use config;
use date_time;

use CDF;
use Thingy;
use Tree;

# -----------------------------------------------------------------------------

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(print_tree);


# -----------------------------------------------------------------------------
# -- MODULE/PACKAGE VARS :
# -----------------------------------------------------------------------------

my $cgi_root = &cgi_root_dir;


# -----------------------------------------------------------------------------
# -- EXPORTS :

sub print_tree() {

  my $cookies  = shift;
  my $doc      = shift;
  my $except   = shift;
  my $s        = shift;

  my $admin    = shift; 
      # ...admin people have special permissions:  they can read/write
      #    everything and delete things to!

    my $TREE     = $s->{TREE};
    my $USERNAME = $s->{USERNAME};
    my $REALNAME = $s->{REALNAME};
    my $BY       = $s->{VIEWTREEBY};

    my @entry_list_list;
    my $cdf;
    my $tree = Tree->new($except, $TREE);

    my $string = "";

    # -- add the root CDF to the tree :
    my $root_cdf = { };

    $root_cdf->{CDF}    = "/";
    $root_cdf->{README} = "--not-needed--";
    $root_cdf->{ALLOW}  = "--not-needed--";
    $root_cdf->{DENY}   = "--not-needed--";

    push (@{$tree->{TREETOO}}, $root_cdf);

    # -- for each CDF in the Tree (including the root) :

    foreach (@{$tree->{TREETOO}}) {

        $cdf = CDF->new($cookies, $doc, $except, $s, $TREE, $_->{CDF});

        # -- if have read-permission get Entries :
        if (($admin eq "am_admin") ||
            ($cdf->read_permission_granted($except, $s))) {

            my $conts       = { };
            $conts->{CONTS} = $cdf->an_sort_cts();
            $conts->{CDF}   = $_->{CDF};

            # -- add to list :
            push(@entry_list_list, $conts);
          }
      }
   
    # -- sort and print :
    #...

    if ($BY eq "revcron") {
    
        # -- get each Thingy's date-stamp :

        my %entry_hash;

        foreach my $e_l_l (sort @entry_list_list) {

            foreach (@{$e_l_l->{CONTS}}) {
 
                my $esc = $_;
                $esc =~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
                    # ...escape the urls...

                my $thingy    = Thingy->down_new($TREE, $e_l_l->{CDF}, $_);
                my $ehk       = $thingy->time . $e_l_l->{CDF} . $_;
                my $date_time = &date_time($thingy->time);

                $entry_hash{$ehk} =
                  "$e_l_l->{CDF}\/" . 
                  "<A HREF=\"$cgi_root/navigate.pl" . 
                            "?NAVIGATE=NAVIGATE" .
                            "&TREE=$TREE&LOCATION=$e_l_l->{CDF}\/" .
                            "&USERNAME=$USERNAME&REALNAME=$REALNAME" . 
                            "&VIEW=ONEONLY" .
                            "&VIEWONEONLY=$esc\">$_</A></TD>" . 
                            "<TD>" .  $date_time;

                if ($admin eq "am_admin") {
                $entry_hash{$ehk} .=
                          "</TD><TD><A HREF=\"admin_entries.pl" . 
                          "?TREE=$TREE&LOCATION=$e_l_l->{CDF}\/" .
                          "&AEACTION=DELETE&ENTRY=$esc\">delete</A>"
                  }
              }
          }

        $string .= "\n<BR><H1>The Tree: $TREE: All Readable Entries in " 
                . "Reverse-Cronological-Order</H1>";

        unless ($admin eq "am_admin") {
            $string .= "\n<CENTER><B><I>entries residing in branches "
                       . "for which you have no read-permission are NOT "
                       . "shown.</I></B></CENTER>\n<BR><BR>";
          }

        sub backwards {$b cmp $a}

        $string .= "\n<TABLE WIDTH=\"100%\">";
        $string .= "\n<TR><TD><B>Entry</B></TD>" . 
                     "<TD><B>Date/Time</B></TD></TR>";
        foreach (sort backwards keys %entry_hash) {
            $string .= "\n<TR><TD>$entry_hash{$_}</TD></TR>";
          }
        $string .= "\n</TABLE>";
      }

    elsif ($BY eq "structure") {

        $string .= "\n<BR><H1>The Tree: $TREE: All Readable Entries by " 
                . "Structure</H1>";

        unless ($admin eq "am_admin") {
            $string .= "\n<B>Nota bene: <I>entries residing in branches "
                       . "for which you have no read-permission are NOT "
                       . "shown.</I></B>\n<BR><BR>";
	  }

        $string .= "\n<TABLE WIDTH=\"100%\">";
        $string .= "\n<TR><TD><B>Entry</B></TD>";
        if ($admin eq "am_admin") {
            $string .= "<TD><B>Delete?</B></TD></TR>";
          }
        else {$string .= "<TD><B> </B></TD></TR>";}


        foreach my $e_l_l (sort @entry_list_list) {

            if (scalar(@{$e_l_l->{CONTS}}) > 0) {
                 $string .= "\n<TR><TD></TD><TD></TD></TR>";
              }

            foreach (@{$e_l_l->{CONTS}}) {

                my $esc = $_;
                $esc =~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
                    # ...escape the urls...

                $string .= "\n<TR><TD>$e_l_l->{CDF}/" .
                      "<A HREF=\"$cgi_root/navigate.pl" . 
                      "?NAVIGATE=NAVIGATE" .
                      "&TREE=$TREE&LOCATION=$e_l_l->{CDF}\/" .
                      "&USERNAME=$USERNAME&REALNAME=$REALNAME" . 
                      "&VIEW=ONEONLY" .
                      "&VIEWONEONLY=$esc\">$_</A></TD>";

                if ($admin eq "am_admin") {
                    $string .= "\n <TD>" .
                          "<A HREF=\"admin_entries.pl" . 
                          "?TREE=$TREE&LOCATION=$e_l_l->{CDF}\/" .
                          "&AEACTION=DELETE&ENTRY=$esc\">delete</A></TD>";
                  }
                $string .= "</TR>";
              }
          }
        $string .= "</TABLE>";
      }

    return $string;
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
