# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------

package CDF;

# -----------------------------------------------------------------------------
# History :
# 
# -- 2000       :  Wrote it.
#
# -- 2001 Mar 27:  Reorganised a bit.
# -----------------------------------------------------------------------------

use strict;
use config;
use cdf_permissions;


# -----------------------------------------------------------------------------
# -- PUBLIC INTERFACE :
#    
#    new --- (read) constructor;
#    
#    children, rec_childs, contents, location, parent, readme, allow, 
#        deny --- library routines which return field values;
#    

# -----------------------------------------------------------------------------
# -- Module/Package variables :

# -- position of document-trees in host OS:
my $root = &root;

# -----------------------------------------------------------------------------
# -- PUBLIC :  CONSTRUCTOR (READ) :
# -----------------------------------------------------------------------------

sub new {                        

  my $class    = shift;           
  my $cookies  = shift;
  my $doc      = shift;
  my $except   = shift;
  my $s        = shift;
  my $tree     = shift;
  my $location = shift;

    my $self = { };              

    my $textline;

    bless($self, $class);             

    $self->{CHILDREN}  = &new_children($tree, $location);
    $self->{RECCHILDS} = &new_rec_childs($tree, $location);

    $self->{ANSORTCTS} = &new_sorted_contents($tree, $location);

    $self->{LOCATION}  = $location;
    $self->{PARENT}    = &new_parent($cookies, $doc, $except, $s);
    $self->{ALLOW}     = &new_allow($cookies, $doc, $except, $s);
    $self->{DENY}      = &new_deny($cookies, $doc, $except, $s);
    $self->{README}    = &new_readme($cookies, $doc, $except, $s);

    return $self;
  }


sub new_2 {                        

  my $class    = shift;           
  my $cookies  = shift;
  my $doc      = shift;
  my $except   = shift;
  my $s        = shift;

    my $tree     = $s->attr("TREE");
    my $location = $s->attr("LOCATION");

    my $self = { };              

    my $textline;

    bless($self, $class);             

    $self->{CHILDREN}  = &new_children($tree, $location);
    $self->{RECCHILDS} = &new_rec_childs($tree, $location);

    $self->{ANSORTCTS} = &new_sorted_contents($tree, $location);

    $self->{LOCATION}  = $location;
    $self->{PARENT}    = &new_parent($cookies, $doc, $except, $s);
    $self->{ALLOW}     = &new_allow($cookies, $doc, $except, $s);
    $self->{DENY}      = &new_deny($cookies, $doc, $except, $s);
    $self->{README}    = &new_readme($cookies, $doc, $except, $s);

    return $self;
  }


# -----------------------------------------------------------------------------
# -- PERMISSIONS :
# -----------------------------------------------------------------------------

# -- 

sub read_permission_granted () {

  my $self   = shift;
  my $except = shift;
  my $s      = shift;

    my $username = $s->attr("USERNAME");
    my $tree     = $s->attr("TREE");
    my $location = $s->attr("LOCATION");

    if (&read_explicitly_denied($except, $username, $tree, $location)) {
        return 0;
      }

    if (&read_groupely_denied($except, $username, $tree, $location)) {
        unless (&read_explicitly_allowed($username, $tree, $location)) {
            return 0;
          }
      }

    if (&read_denied($except, $username, $tree, $location)) {
        unless (&read_explicitly_allowed($except, 
                                         $username, $tree, $location) ||
                &read_groupely_allowed($except, 
                                       $username, $tree, $location)     ) {
            return 0;
          }
      }

    unless(&read_allowed($except, $username, $tree, $location)) {
        return 0;
      }

    return 1;
  }


# -----------------------------------------------------------------------------
# --

sub write_permission_granted () {

  my $self     = shift;
  my $except   = shift;
  my $username = shift;
  my $tree     = shift;
  my $location = shift;

    if (&write_explicitly_denied($except, $username, $tree, $location)) {
        return 0;
      }

    if (&write_groupely_denied($except, $username, $tree, $location)) {
        unless (&write_explicitly_allowed($except, 
                                          $username, $tree, $location)) {
            return 0;
          }
      }

    if (&write_denied($except, $username, $tree, $location)) {
        unless (&write_explicitly_allowed($except, 
                                          $username, $tree, $location) ||
               (&write_groupely_allowed($except, $username, $tree, $location))) {
            return 0;
          }
      }

    unless(&write_allowed($except, $username, $tree, $location)) {
        return 0;
      }

    return 1;
  }


# -----------------------------------------------------------------------------
# -- PUBLIC :  READ-FIELD ROUTINES :
# -----------------------------------------------------------------------------

sub children {
    my $self = shift;
    return $self->{CHILDREN};
  }

sub rec_childs {
    my $self = shift;
    return $self->{RECCHILDS};
  }

sub an_sort_cts {
    my $self = shift;
    return $self->{ANSORTCTS};
  }

sub location {
    my $self = shift;
    return $self->{LOCATION};
  }

sub parent {
    my $self = shift;
    return $self->{PARENT};
  }

sub readme {
    my $self = shift;
    return $self->{README};
  }

sub allow {
    my $self = shift;
    return $self->{ALLOW};
  }

sub deny {
    my $self = shift;
    return $self->{DENY};
  }

# -----------------------------------------------------------------------------
# -- PRIVATE :  CONSTRUCTOR SLAVES :
# -----------------------------------------------------------------------------

sub new_readme () {

  my $cookies  = shift;
  my $doc      = shift;
  my $except   = shift;
  my $s        = shift;

    my $tree     = $s->attr("TREE");
    my $location = $s->attr("LOCATION");

    my ($readme, $textline);

    open(README, "$root$tree$location/" . "README.node") 
        || $except->raise_logout_2($cookies, $doc, $s,
                                   "Can't open README.node: " . 
                                   "$location/README.node");
    while ($textline = <README>) {
        $readme .= $textline;
      }
    close(README);

    return $readme;
  }

# -----------------------------------------------------------------------------

sub new_allow () {

  my $cookies  = shift;
  my $doc      = shift;
  my $except   = shift;
  my $s        = shift;

    my $tree     = $s->attr("TREE");
    my $location = $s->attr("LOCATION");

    my ($allow, $textline);

    open(ALLOW, "$root$tree$location/" . "USERS.ALLOW") 
        || $except->raise_logout_2($cookies, $doc, $s,
                                   "Can't open USERS.ALLOW: " . 
                                   "$location/USERS.ALLOW");
    while ($textline = <ALLOW>) {
        $allow .= $textline;
      }
    close(ALLOW);

    return $allow;
  }

# -----------------------------------------------------------------------------

sub new_deny () {

  my $cookies  = shift;
  my $doc      = shift;
  my $except   = shift;
  my $s        = shift;

    my $tree     = $s->attr("TREE");
    my $location = $s->attr("LOCATION");

    my ($deny, $textline);

    open(DENY, "$root$tree$location/" . "USERS.DENY") 
        || $except->raise_logout_2($cookies, $doc, $s,
                                   "Can't open USERS.DENY: " . 
                                   "$location/USERS.DENY");
    while ($textline = <DENY>) {
        $deny .= $textline;
      }
    close(DENY);

    return $deny;
  }

# -----------------------------------------------------------------------------

sub new_parent () {

  my $cookies  = shift;
  my $doc      = shift;
  my $except   = shift;
  my $s        = shift;

    my $tree     = $s->attr("TREE");
    my $location = $s->attr("LOCATION");

    if (($location eq "/") || ($location eq "//")) {
        return &no_parent;
      }
    else {
        $location =~ s/\/$//;
        $location =~ s/[^\/]*$//;
        return $location;
      }
  }

# -----------------------------------------------------------------------------

sub new_rec_childs () {

  my $tree = shift @_;
  my $path = shift @_;

    my @branches = ();

    my $root = &root;
    my $filename;

    opendir (DIR, "$root/$tree/$path")
        || die "Can't open DIR: $tree/$path";


    foreach $filename (sort readdir(DIR)) {

        if ($filename eq ".") {next}
        if ($filename eq "..") {next}

        open(FILE, "$root$tree$path/$filename") 
            || die "Can't open FILE: $root$tree$path/$filename";

        if(-d FILE) {
            push (@branches, "$path/$filename");
            my $branches = &new_rec_childs($tree, "$path/$filename");
            push (@branches, @$branches);
          }
        else {
            close(FILE); 
            next;
         } 
      }

    closedir (DIR);

    return \@branches;
  }

# -----------------------------------------------------------------------------

sub new_children () {

  my $tree     = shift @_;
  my $location = shift @_;

    my ($child);
    my @children = ();

    opendir(CHILDREN, $root . $tree . $location)
        || die "Can't open CHILDREN at location: $location";

    while ($child = readdir(CHILDREN)) {
        if ($child eq ".") {next}
        if ($child eq "..") {next}
        open(FILE, "$root$tree$location/$child") 
            || die "Can't open FILE: $root$tree$location/$child";
        unless (-d FILE) {next}
        close(FILE);
        push(@children, $child);
      }

    closedir(CHILDREN);
    return \@children;
  }

# -----------------------------------------------------------------------------
# -- return lists of THINGYs in a CDF: sorted by both alpha-numeric order
#    and reverse-chronological too...

sub new_sorted_contents () {

  my $tree      = shift @_;
  my $location  = shift @_;

    my ($cont);
    my @contents = ();

    opendir(CONTENTS, $root . $tree . $location)
        || die "Can't open CONTENTS at location: $root$tree$location";

    while ($cont = readdir(CONTENTS)) {
        if ($cont eq ".") {next}
        if ($cont eq "..") {next}
        if ($cont eq "README.node") {next}
        if ($cont eq "USERS.ALLOW") {next}
        if ($cont eq "USERS.DENY") {next}
        open(FILE, "$root$tree$location/$cont") 
            || die "Can't open FILE: $tree$location/$cont";
          if (-d FILE) {next}
        close(FILE);
        if ($cont =~ m/\-\-THINGY$/) {
            $cont =~ s/\-\-THINGY$//;
            push(@contents, $cont)
          }
      }

    closedir(CONTENTS);

    @contents = sort {lc($a) cmp lc($b)} @contents;

    return \@contents;
  }

# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
