package Thingy;

# -----------------------------------------------------------------------------
# Module to create/read Entries in the database.
#    
# History :
#    
#  -- 2000        :  Done.
#    
#  -- 2001 Mar 27 :  Made pretty.
# -----------------------------------------------------------------------------

use strict;
use config;

use Time::Local;

# -----------------------------------------------------------------------------
# -- PUBLIC INTERFACE :
#    
#    constructors: down_new (read an Entry), up_new (creating/writing);
#        
#    author, time, uploader, readme, history, definitive, keywords, 
#        viewonly --- routines to return field values;
#        
#    write, write_nv --- routines to write a new Entry and a new version of
#        an Entry (that have been created by the up_new constructor);

    
# -----------------------------------------------------------------------------
# -- MODULE/PACKAGE VARS:

my $root         = &root;
my $cgi_root_dir = &cgi_root_dir;

my $THINGY = "";
my @THINGY;


# -----------------------------------------------------------------------------
# -- PUBLIC CONSTRUCTORS :
# -----------------------------------------------------------------------------

# -- 3 supercedes 2 by using exceptions rather than die :

sub down_new_3 {                       

  my $class  = shift;          
  my $except = shift;
  my $s      = shift;

    my $tree     = $s->{TREE};
    my $location = $s->{LOCATION};
    my $thingy   = $s->{THINGY};

    my $self = { };             

    my ($textline);

    bless($self, $class);       

    $THINGY = "";


    open(THINGY, "$root$tree$location/$thingy--THINGY") 
        || die "Can't open thingy: $root$tree$location$thingy";
#############################                   ........raise_logout_3........

    while($textline = <THINGY>) {
        $THINGY .= $textline;
      }
    close(THINGY);

    @THINGY = split(/\n/, $THINGY);

    $self->{NAME}       = $thingy;
    $self->{DEFINITIVE} = &new_definitive($tree, $location, $thingy);
    $self->{VIEWONLY}   = &new_viewonly($tree, $location, $thingy);
    $self->{AUTHOR}     = &new_author($tree, $location);

    ($self->{TIME}, $self->{DATEWRIT}) = &new_times($tree, $location);

    $self->{UPLOADER}   = &new_uploader($tree, $location);
    $self->{README}     = &new_readme($tree, $location);
    $self->{HISTORY}    = &new_history($tree, $location);
    $self->{KEYWORDS}   = &new_keywords();

    return $self;
  }


sub down_new_2 {                       

  my $class = shift;          
  my $s     = shift;

    my $tree     = $s->{TREE};
    my $location = $s->{LOCATION};
    my $thingy   = $s->{THINGY};

    my $self = { };             

    my ($textline);

    bless($self, $class);       

    $THINGY = "";


    open(THINGY, "$root$tree$location/$thingy--THINGY") 
        || die "Can't open thingy: $root$tree$location$thingy";
##############################                   ........raise_logout_3........

    while($textline = <THINGY>) {
        $THINGY .= $textline;
      }
    close(THINGY);

    @THINGY = split(/\n/, $THINGY);

    $self->{NAME}       = $thingy;
    $self->{DEFINITIVE} = &new_definitive($tree, $location, $thingy);
    $self->{VIEWONLY}   = &new_viewonly($tree, $location, $thingy);
    $self->{AUTHOR}     = &new_author($tree, $location);

    ($self->{TIME}, $self->{DATEWRIT}) = &new_times($tree, $location);

    $self->{UPLOADER}   = &new_uploader($tree, $location);
    $self->{README}     = &new_readme($tree, $location);
    $self->{HISTORY}    = &new_history($tree, $location);
    $self->{KEYWORDS}   = &new_keywords();

    return $self;
  }

sub down_new {                       

  my $class    = shift;          
  my $tree     = shift;
  my $location = shift;
  my $thingy   = shift;

    my $self   = { };             

    my ($textline);

    bless($self, $class);       

    $THINGY = "";


    open(THINGY, "$root$tree$location/$thingy--THINGY") 
        || die "Can't open thingy: $root$tree$location$thingy";
    while($textline = <THINGY>) {
        $THINGY .= $textline;
      }
    close(THINGY);

    @THINGY = split(/\n/, $THINGY);

    $self->{NAME}       = $thingy;
    $self->{DEFINITIVE} = &new_definitive($tree, $location, $thingy);
    $self->{VIEWONLY}   = &new_viewonly($tree, $location, $thingy);
    $self->{AUTHOR}     = &new_author($tree, $location);

    ($self->{TIME}, $self->{DATEWRIT}) = &new_times($tree, $location);

    $self->{UPLOADER}   = &new_uploader($tree, $location);
    $self->{README}     = &new_readme($tree, $location);
    $self->{HISTORY}    = &new_history($tree, $location);
    $self->{KEYWORDS}   = &new_keywords();

    return $self;
  }


# -----------------------------------------------------------------------------

sub up_new {                       

  my $class    = shift;          

  my $tree     = shift;
  my $location = shift;

  my $doc_def    = shift;
  my $doc_view   = shift;

  my $doc_keyws  = shift;
  my $doc_auth   = shift;

  my $doc_uploader  = shift;
  my $doc_date_writ = shift;

    # -- convert date to Epoch-Seconds :
    my ($dd, $mm, $yyyy);
    ($dd, $mm, $yyyy) = split(/\//, $doc_date_writ);
    $doc_date_writ = timelocal(0, 0, 0, $dd, $mm-1, $yyyy-1900);

    my $self   = { };             

    bless($self, $class);       

    if ($doc_def =~ m/\S/) {
        $self->{DEFINITIVE} = $doc_def
      }
    else {$self->{DEFINITIVE} = "none"}

    if ($doc_view =~ m/\S/) {
        $self->{VIEWONLY} = $doc_view
      }
    else {$self->{VIEWONLY} = "none"}

    $self->{KEYWORDS} = $doc_keyws;
    $self->{AUTHOR}   = $doc_auth;
    $self->{UPLOADER} = $doc_uploader;
    $self->{DATEWRIT} = $doc_date_writ;

    return $self;
  }


# -----------------------------------------------------------------------------
# -- PUBLIC :  ROUTINES TO RETURN VALUES :
# -----------------------------------------------------------------------------

sub name {
    my $self = shift;
    return $self->{NAME};
  }

sub author {
    my $self = shift;
    return $self->{AUTHOR};
  }

sub datewrit {
    my $self = shift;
    return $self->{DATEWRIT};
  }

sub time {
    my $self = shift;
    return $self->{TIME};
  }

sub uploader {
    my $self = shift;
    return $self->{UPLOADER};
  }

sub readme {
    my $self = shift;
    return $self->{README};
  }

sub history {
    my $self = shift;
    return $self->{HISTORY};
  }

sub definitive {
    my $self = shift;
    return $self->{DEFINITIVE};
  }

sub keywords {
    my $self = shift;
    return $self->{KEYWORDS};
  }

sub viewonly {
    my $self = shift;
    return $self->{VIEWONLY};
  }


# -----------------------------------------------------------------------------
# -- PUBLIC LIB ROUTINES FOR WRITING NEWLY CREATED (WITH CONSTRUCTOR) ENTRIES :
# -----------------------------------------------------------------------------

# -- write new version of exisiting thingy (append README with new info;
#    upload new versions of definitive and viewonly files;  change symbolic
#    links) :

sub write_nv () {
  my $self   = shift;
  my $doc    = shift;
  my $except = shift;

  my $tree        = shift;
  my $location    = shift;
  my $thingy_name = shift;
  my $time        = shift;
  my $hist_append = shift;

    # -- overwrite old THINGY file :

    open (THINGY, ">$root$tree$location/$thingy_name--THINGY") 
        || $except->raise_integrity($doc, 
                                    "Can't write thingy:  " . 
                                    "$tree$location/$thingy_name");

    print THINGY "<THINGY>\n";
    print THINGY "<DEFINITIVE>$self->{DEFINITIVE}</DEFINITIVE>\n";
    print THINGY "<VIEWONLY>$self->{VIEWONLY}</VIEWONLY>\n";
    print THINGY "<TIME>$time</TIME>\n";
    print THINGY "<KEYWORDS>$self->{KEYWORDS}</KEYWORDS>\n";
    print THINGY "<AUTHOR>$self->{AUTHOR}</AUTHOR>\n";
    print THINGY "<UPLOADER>$self->{UPLOADER}</UPLOADER>\n";
    print THINGY "<DATEWRIT>$self->{DATEWRIT}</DATEWRIT>\n";
    print THINGY "<README>$thingy_name.README</README>\n";
    print THINGY "<HISTORY>$thingy_name--HISTORY</HISTORY>\n";
    print THINGY "</THINGY>\n";

    close(THINGY);


    # -- append new info to existing README :
    
    open (HISTORY, ">>$root$tree$location/$thingy_name--HISTORY") 
        || $except->raise_integrity($doc,
                                    "Can't write HISTORY:  " . 
                                    "$tree$location/$thingy_name");

    print HISTORY "\n<HISTORY>";
    print HISTORY "\n<DATEWRIT>$self->{DATEWRIT}</DATEWRIT>";
    print HISTORY "\n<DATE>$time</DATE>";
    print HISTORY "\n<BR>";

    if ($self->{DEFINITIVE} eq "none") {
        print HISTORY "\n<DEFINITIVE></DEFINITIVE>";
      }
    else {
        print HISTORY "\n<DEFINITIVE>"
            . "<QSLINK>$thingy_name--$time--$self->{DEFINITIVE}</QSLINK>"
            . "</DEFINITIVE>";
      }

    if ($self->{VIEWONLY} eq "none") {
        print HISTORY "\n<VIEWONLY></VIEWONLY>";
      }
    else {
        print HISTORY "\n<VIEWONLY>"
            . "<QSLINK>$thingy_name--$time--$self->{VIEWONLY}</QSLINK>"
            . "</VIEWONLY>";
      }

    print HISTORY "\n<BR>";

    print HISTORY "\n<KEYWORDS>$self->{KEYWORDS}</KEYWORDS>";
    print HISTORY "\n<AUTHOR>$self->{AUTHOR}</AUTHOR>";
    print HISTORY "\n<UPLOADER>$self->{UPLOADER}</UPLOADER>";

    print HISTORY "\n<BR>";

    print HISTORY "\n<README>$hist_append</README>";

    print HISTORY "\n</HISTORY>\n";
    close(HISTORY);


    # -- update symbolic links to current definitive and viewonly doc files :

    chdir("$root$tree$location");


    # -- if we have a new definitive file, reset the symbolic link :
    if (-e "$root$tree$location$thingy_name--$time--$self->{DEFINITIVE}") {

        $doc->append("resetting symbolic link: " . 
                     "$thingy_name--$self->{DEFINITIVE}<BR>\n");

        if (-e "$thingy_name--$self->{DEFINITIVE}") {
            unlink("$thingy_name--$self->{DEFINITIVE}")
                || $except->raise_integrity($doc, 
                                            "Can't unlink definitive doc: " . 
                          "$tree$location$thingy_name--$self->{DEFINITIVE}");
	  }

        symlink("$thingy_name--$time--$self->{DEFINITIVE}", 
                "$thingy_name--$self->{DEFINITIVE}")
            || $except->raise_integrity($doc,
                                        "Unable to create symlink: " . 
                   "$tree$location$thingy_name--$self->{DEFINITIVE}");
      }
    

    if (-e "$root$tree$location$thingy_name--$time--$self->{VIEWONLY}") {

        $doc->append("resetting symbolic link: " . 
                     "$thingy_name--$self->{VIEWONLY}<BR>\n");

        if (-e "$thingy_name--$self->{VIEWONLY}") {
            unlink("$thingy_name--$self->{VIEWONLY}")
                || die "Can't unlink viewonly doc: " . 
                       "$tree$location$thingy_name--$self->{VIEWONLY}";
	  }

        symlink("$thingy_name--$time--$self->{VIEWONLY}", 
                "$thingy_name--$self->{VIEWONLY}")
            || die "Unable to create symlink: " . 
                   "$tree$location$thingy_name--$self->{VIEWONLY}";
      }

    # -- tell customer what we done :

    $doc->append("<H3>New entry:</H3>\n");
    
    $doc->append("<PRE>\n");
    $doc->append("Definitive : $self->{DEFINITIVE}\n");
    $doc->append("Viewonly   : $self->{VIEWONLY}\n");
    $doc->append("Author     : $self->{AUTHOR}\n");
    $doc->append("Uploader   : $self->{UPLOADER}\n");
    $doc->append("Keywords   : $self->{KEYWORDS}\n");
    $doc->append("Readme     : $thingy_name.README\n");
    $doc->append("</PRE>\n");

    $doc->append("...with README...");
    $doc->append("<PRE>\n");
    $doc->append("$self->{README}\n");
    $doc->append("</PRE>\n");
  }

# -----------------------------------------------------------------------------

sub write {
  my $self = shift;
  my $doc  = shift;

  my $tree        = shift;
  my $location    = shift;
  my $thingy_name = shift;
  my $time        = shift;
  my $readme      = shift;

    open (THINGY, ">$root$tree$location/$thingy_name--THINGY") 
        || die "Can't write thingy:  $tree$location/$thingy_name";

    print THINGY "<THINGY>\n";
    print THINGY "<DEFINITIVE>$self->{DEFINITIVE}</DEFINITIVE>\n";
    print THINGY "<VIEWONLY>$self->{VIEWONLY}</VIEWONLY>\n";
    print THINGY "<TIME>$time</TIME>\n";
    print THINGY "<KEYWORDS>$self->{KEYWORDS}</KEYWORDS>\n";
    print THINGY "<AUTHOR>$self->{AUTHOR}</AUTHOR>\n";
    print THINGY "<UPLOADER>$self->{UPLOADER}</UPLOADER>\n";
    print THINGY "<DATEWRIT>$self->{DATEWRIT}</DATEWRIT>\n";
    print THINGY "<README>$thingy_name.README</README>\n";
    print THINGY "<HISTORY>$thingy_name--HISTORY</HISTORY>\n";
    print THINGY "</THINGY>\n";

    close(THINGY);

    open (README, ">$root$tree$location/$thingy_name.README") 
        || die "Can't write HISTORY:  $tree$location/$thingy_name";

    print README $readme;

    close (README);

    open (HISTORY, ">$root$tree$location/$thingy_name--HISTORY") 
        || die "Can't write readme:  $tree$location/$thingy_name";

    print HISTORY "\n<HISTORY>";
    print HISTORY "\n<DATEWRIT>$self->{DATEWRIT}</DATEWRIT>";
    print HISTORY "\n<DATE>$time</DATE>";
    print HISTORY "\n<BR>";

    if ($self->{DEFINITIVE} eq "none") {
        print HISTORY "\n<DEFINITIVE></DEFINITIVE>";
      }
    else {
        print HISTORY "\n<DEFINITIVE>"
            . "<QSLINK>$thingy_name--$time--$self->{DEFINITIVE}</QSLINK>" 
            . "</DEFINITIVE>";
      }

    if ($self->{VIEWONLY} eq "none") {
        print HISTORY "\n<VIEWONLY></VIEWONLY>";
      }
    else {
        print HISTORY "\n<VIEWONLY>"
            . "<QSLINK>$thingy_name--$time--$self->{VIEWONLY}</QSLINK>"
            . "</VIEWONLY>";
      }

    print HISTORY "\n<BR>";

    print HISTORY "\n<KEYWORDS>$self->{KEYWORDS}</KEYWORDS>";
    print HISTORY "\n<AUTHOR>$self->{AUTHOR}</AUTHOR>";
    print HISTORY "\n<UPLOADER>$self->{UPLOADER}</UPLOADER>";

    print HISTORY "\n<BR>";

    print HISTORY "\n<README>$readme</README>";

    print HISTORY "\n</HISTORY>\n";
    close(HISTORY);

    $doc->append("<H3>New entry:</H3>\n");
    
    $doc->append("<PRE>\n");
    $doc->append("Definitive : $self->{DEFINITIVE}\n");
    $doc->append("Viewonly   : $self->{VIEWONLY}\n");
    $doc->append("Author     : $self->{AUTHOR}\n");
    $doc->append("Uploader   : $self->{UPLOADER}\n");
    $doc->append("Keywords   : $self->{KEYWORDS}\n");
    ##print "Readme     : $thingy_name.README\n";
    ##print "History    : $thingy_name--HISTORY\n";
    $doc->append("</PRE>\n");

    $doc->append("<B>...with README...</B>");
    $doc->append("<PRE>\n");
    $doc->append("$readme\n");
    $doc->append("</PRE>\n");
  }


# -----------------------------------------------------------------------------
# -- LOCAL :  CONSTRUCTOR SLAVES :
# -----------------------------------------------------------------------------

sub new_definitive() {

  my $tree     = shift @_;
  my $location = shift @_;
  my $thingy   = shift @_;

    my $definitive_file = "none";

    my @TC = @THINGY;

    my ($textline, @def);
    my ($size, $atime, $mtime, $ctime);

    while ($textline = pop(@TC)) {
        if ($textline =~ m/^\s*<DEFINITIVE>/) {
            $definitive_file = $textline;
          }
        $definitive_file =~ s/<\/?DEFINITIVE>//gs;
      }

    open (DEF, "$root$tree$location/$thingy--$definitive_file");
    if (-e DEF) {
        ($size, $atime, $mtime, $ctime) 
            = (stat("$root$tree$location/$thingy--$definitive_file")) [7,10];
      }
    else {
        $size = "???";  $atime = "???";  $mtime = "???";  $ctime = "???"; 
      }
    close(DEF);

    @def = ($definitive_file, $size, $atime, $mtime, $ctime);

    return \@def;
  }

# -----------------------------------------------------------------------------

sub new_viewonly() {

  my $tree     = shift @_;
  my $location = shift @_;
  my $thingy   = shift @_;

    my $viewonly_file;

    my @TC = @THINGY;

    my ($textline, @view);
    my ($size, $atime, $mtime, $ctime);

    while ($textline = pop(@TC)) {
        if ($textline =~ m/^\s*<VIEWONLY>/) {
            $viewonly_file = $textline;
          }
        $viewonly_file =~ s/<\/?VIEWONLY>//gs;
      }

    open (VIEW, "$root$tree$location/$thingy--$viewonly_file");
    if (-e VIEW) {
        ($size, $atime, $mtime, $ctime) 
            = (stat("$root$tree$location/$thingy--$viewonly_file")) [7,10];
      }
    else {
        $size = "???";  $atime = "???";  $mtime = "???";  $ctime = "???"; 
      }
    close(VIEW);

    @view = ($viewonly_file, $size, $atime, $mtime, $ctime);

    return \@view;
  }

# -----------------------------------------------------------------------------

sub new_readme() {

  my $tree     = shift @_;
  my $location = shift @_;

    my $readme_file = "";
    my $readme      = "";

    my @TC = @THINGY;

    my $textline;

    while ($textline = pop(@TC)) {
        if ($textline =~ m/^\s*<README>/) {
            $readme_file = $textline;
            $readme_file =~ s/<\/?README>//gs;

            if (open(README, "$root$tree$location/$readme_file")) {
                while ($textline = <README>) {
                    $readme .= $textline;
                  }
                close(README);
              }
            else {
                $readme = "[[Can't open: $root$tree$location/$readme_file]]";
              }
            return $readme;
          }
      }
  }

# -----------------------------------------------------------------------------

sub new_history() {

  my $tree     = shift @_;
  my $location = shift @_;

    my $history_file = "";
    my $history      = "";

    my @TC = @THINGY;

    my $textline;

    while ($textline = pop(@TC)) {
        if ($textline =~ m/^\s*<HISTORY>/) {
            $history_file = $textline;
            $history_file =~ s/<\/?HISTORY>//gs;

            if (open(HISTORY, "$root$tree$location/$history_file")) {
                while ($textline = <HISTORY>) {
                    $history .= $textline;
                  }
                close(HISTORY);
              }
            else {
                $history = "[[Can't open: $root$tree$location/$history_file]]";
              }
            return $history;
          }
      }
  }

# -----------------------------------------------------------------------------
# -- older entries will not have a DATEWRIT, only upload TIME, so revert
#    to this if necessary :

sub new_times() {

    my $time = 0;
    my $datewrit = 0;

    my @TC = @THINGY;

    my $textline = "";
#    my $state    = "no";

    while ($textline = pop(@TC)) {
        if (($textline =~ m/^\s*<TIME>/)) {
            $time .= $textline;
#            $state = "yes";
            if ($textline =~ m/<\/TIME>/) {
                $time =~ s/<\/?TIME>//gs;
              }
          }

        if (($textline =~ m/^\s*<DATEWRIT>/)) {
            $datewrit .= $textline;
#            $state = "yes";
            if ($textline =~ m/<\/DATEWRIT>/) {
                $datewrit =~ s/<\/?DATEWRIT>//gs;
              }
          }
      }

    unless ($datewrit > 0) {$datewrit = $time};
    return ($time, $datewrit);
  }

# -----------------------------------------------------------------------------

sub new_author() {

    my $author;

    my @TC = @THINGY;

    my $textline = "";
    my $state    = "no";

    while ($textline = pop(@TC)) {
        if (($textline =~ m/^\s*<AUTHOR>/) || ($state eq "yes")) {
            $author .= $textline;
            $state = "yes";
            if ($textline =~ m/<\/AUTHOR>/) {
                $author =~ s/<\/?AUTHOR>//gs;
                return $author;
              }
          }
      }
  }

# -----------------------------------------------------------------------------

sub new_uploader() {

    my $uploader;

    my @TC = @THINGY;

    my $textline = "";
    my $state    = "no";

    while ($textline = pop(@TC)) {
        if (($textline =~ m/^\s*<UPLOADER>/) || ($state eq "yes")) {
            $uploader .= $textline;
            $state = "yes";
            if ($textline =~ m/<\/UPLOADER>/) {
                $uploader =~ s/<\/?UPLOADER>//gs;
                return $uploader;
              }
          }
      }
  }

# -----------------------------------------------------------------------------

sub new_keywords() {

    my @keywords;

    my @TC = @THINGY;

    my $keywords = "";
    my $textline = "";
    my $state    = "no";

    while ($textline = pop(@TC)) {
        if (($textline =~ m/^\s*<KEYWORDS>/) || ($state eq "yes")) {
            $keywords .= $textline;
            $state = "yes";
            if ($textline =~ m/<\/KEYWORDS>/) {
                $keywords =~ s/<\/?KEYWORDS>//gs;
                @keywords = split(/\s/, $keywords);
                return \@keywords;
              }
          }
      }
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
