# -----------------------------------------------------------------------------
package cdf_permissions;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2000 Dec 08: 
#          
# -- 
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

use config;

use Passwd;

# -----------------------------------------------------------------------------

require Exporter;

# -----------------------------------------------------------------------------

@ISA    = qw(Exporter);
@EXPORT = qw(read_explicitly_allowed
             read_groupely_allowed    
             read_allowed
             read_explicitly_denied
             read_groupely_denied
             read_denied
             write_explicitly_allowed
             write_groupely_allowed
             write_allowed
             write_explicitly_denied
             write_groupely_denied
             write_denied
             );

# -----------------------------------------------------------------------------

my $root         = &root;
my $cgi_root_dir = &cgi_root_dir;


# -----------------------------------------------------------------------------
# -- LOCAL :
# -----------------------------------------------------------------------------

sub read_explicitly_allowed () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $allowed = 0;

    open(ALLOW, "$root$tree$location" . "USERS.ALLOW") 
        || $except->raise_options("Can't open USERS.ALLOW");

    while($textline = <ALLOW>) {
        ($one, $two) = split(/\s+/, $textline);
        if (($one eq $username) && ($two =~ m/R/i)) {$allowed = 1}
      }

    close(ALLOW);    return $allowed;
  }

# -----------------------------------------------------------------------------

sub read_groupely_allowed () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $allowed = 0;

    my $passwd = Passwd->new();

    open(ALLOW, "$root$tree$location" . "USERS.ALLOW") 
        || $except->raise_options("Can't open USERS.ALLOW");

    while($textline = <ALLOW>) {
        ($one, $two) = split(/\s+/, $textline);
        if (($one eq $username) && ($two =~ m/R/i)) {
            $allowed = 1;
            print "\n<BR> am userly allowed!";
          }
        elsif (($one =~ m/^GROUP_/) && 
               ($passwd->in_group($except, $username, $one)) &&
               ($two =~ m/R/i)) {
            $allowed = 1;
          }
      }
    close(ALLOW);    return $allowed;
  }

# -----------------------------------------------------------------------------

sub read_allowed () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $allowed = 0;

    my $passwd = Passwd->new();

    open(ALLOW, "$root$tree$location" . "USERS.ALLOW") 
        || $except->raise_options("Can't open USERS.ALLOW");

    while($textline = <ALLOW>) {
        ($one, $two) = split(/\s+/, $textline);        

        if (($one eq $username) && ($two =~ m/R/i)) {
            $allowed = 1;
          }
        elsif (($one eq "ALL") && ($two =~ m/R/i)) {
            $allowed = 1;
          }
        elsif (($one =~ m/^GROUP_/) && 
               ($passwd->in_group($except, $username, $one)) && 
               ($two =~ m/R/i)                 ) {
            $allowed = 1;
          }
      }

    close(ALLOW);    return $allowed;
  }

# -----------------------------------------------------------------------------

sub read_explicitly_denied () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $denied = 0;

    open(DENY, "$root$tree$location" . "USERS.DENY") 
        || $except->raise_options("Can't open $location" . "USERS.DENY");

    while($textline = <DENY>) {
        ($one, $two) = split(//, $textline);        
        if (($one eq $username) && ($two =~ m/R/i)) {$denied = 1}
      }

    close(DENY);    return $denied;
  }

# -----------------------------------------------------------------------------

sub read_groupely_denied () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $denied = 0;

    my $passwd = Passwd->new();

    open(DENY, "$root$tree$location" . "USERS.DENY") 
        || $except->raise_options("Can't open $location" . "USERS.DENY");

    while($textline = <DENY>) {
        ($one, $two) = split(//, $textline);        
        if ( ( ($one eq $username) || 
               (($one =~ m/^GROUP\_/) && 
                ($passwd->in_group($except, $username, $one)))) && 
             ($two =~ m/R/i)) {
            $denied = 1
          }
      }
    close(DENY);    return $denied;
  }

# -----------------------------------------------------------------------------

sub read_denied () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $denied = 0;

    open(DENY, "$root$tree$location" . "USERS.DENY") 
        || $except->raise_options("Can't open USERS.DENY");

    while($textline = <DENY>) {
        ($one, $two) = split(//, $textline);        
        if (    (($one eq $username) || ($one eq "ALL"))
             && ($two =~ m/R/i)) {$denied = 1}
      }

    close(DENY);
    return $denied;
  }


# -----------------------------------------------------------------------------
# -- LOCAL :
# -----------------------------------------------------------------------------

sub write_explicitly_allowed () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $allowed = 0;

    open(ALLOW, "$root$tree$location" . "USERS.ALLOW") 
        || $except->raise_options("Can't open USERS.ALLOW");

    while($textline = <ALLOW>) {
        ($one, $two) = split(/\s+/, $textline);
        if (($one eq $username) && ($two =~ m/W/i)) {$allowed = 1}
      }

    close(ALLOW);    return $allowed;
  }

# -----------------------------------------------------------------------------

sub write_groupely_allowed () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $allowed = 0;

#print "\n<BR> Checking if write_groupely_allowed:";

    my $passwd = Passwd->new();

    open(ALLOW, "$root$tree$location" . "USERS.ALLOW") 
        || $except->raise_options("Can't open USERS.ALLOW");

    while($textline = <ALLOW>) {
        ($one, $two) = split(/\s+/, $textline);
        if (($one eq $username) && ($two =~ m/W/i)) {
            $allowed = 1;
#print "\n<BR> am userly allowed!";
          }
        elsif (($one =~ m/^GROUP_/) && 
               ($passwd->in_group($except, $username, $one)) &&
               ($two =~ m/W/i)) {
            $allowed = 1;
#print "\n<BR> Am groupely allowed!";
          }
      }

    close(ALLOW);
    return $allowed;
  }

# -----------------------------------------------------------------------------

sub write_allowed () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $allowed = 0;

    my $passwd = Passwd->new();

    open(ALLOW, "$root$tree$location" . "USERS.ALLOW") 
        || $except->raise_options("Can't open USERS.ALLOW");

    while($textline = <ALLOW>) {
        ($one, $two) = split(/\s+/, $textline);        
        if (($one eq $username) && ($two =~ m/W/i)) {
#print "\n<BR>Am explicitly read_allowed";
            $allowed = 1;
          }
        elsif (($one eq "ALL") && ($two =~ m/W/i)) {
#print "\n<BR>Am (ALL) read_allowed";
            $allowed = 1;
          }
        elsif (($one =~ m/^GROUP_/) && 
               ($passwd->in_group($except, $username, $one)) && 
               ($two =~ m/W/i)                 ) {
#print "\n<BR>Am groupely write_allowed";
            $allowed = 1;
          }
      }

    close(ALLOW);
    return $allowed;
  }

# -----------------------------------------------------------------------------

sub write_explicitly_denied () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $denied = 0;

    open(DENY, "$root$tree$location" . "USERS.DENY") 
        || $except->raise_options("Can't open USERS.DENY");

    while($textline = <DENY>) {
        ($one, $two) = split(//, $textline);        
        if (($one eq $username) && ($two =~ m/W/i)) {$denied = 1}
      }

    close(DENY);
    return $denied;
  }

# -----------------------------------------------------------------------------

sub write_groupely_denied () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $denied = 0;

    my $passwd = Passwd->new();

    open(DENY, "$root$tree$location" . "USERS.DENY") 
        || $except->raise_options("Can't open USERS.DENY");

    while($textline = <DENY>) {
        ($one, $two) = split(//, $textline);        
        if (($one eq $username) && ($two =~ m/W/i)) {
            $denied = 1;
          }
        elsif (($one =~ m/^GROUP\_/) && 
               ($passwd->in_group($except, $username, $one)) && 
               ($two =~ m/W/i)) {
          }
      }
    close(DENY);
    return $denied;
  }

# -----------------------------------------------------------------------------

sub write_denied () {

  my $except   = shift;
  my $username = shift @_;
  my $tree     = shift @_;
  my $location = shift @_;

    my ($textline, $one, $two);
    my $denied = 0;

    open(DENY, "$root$tree$location" . "USERS.DENY") 
        || $except->raise_options("Can't open USERS.DENY");

    while($textline = <DENY>) {
        ($one, $two) = split(//, $textline);        
        if (    (($one eq $username) || ($one eq "ALL"))
             && ($two =~ m/W/i)) {$denied = 1}
      }

    close(DENY);
    return $denied;
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
