# -----------------------------------------------------------------------------
package sea_mod;
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# History :
# 
# -- 2001 Jan 26: 
#          
# -- 
# 
# -----------------------------------------------------------------------------

use strict;
use vars qw(@ISA @EXPORT);

# -----------------------------------------------------------------------------

require Exporter;

# -----------------------------------------------------------------------------

use config;
use date_time;
use swish_related;
use thingies;

# -----------------------------------------------------------------------------

@ISA    = qw(Exporter);
@EXPORT = qw(
             check_date_fields 
             check_author_fields

             constraint_only_search
             keyword_search

             grep_entry_names
             grep_file_names

             swish_branch_readmes
             swish_entry_readmes
             );


# -----------------------------------------------------------------------------
# -- LOOK-UP TABLES :

my %months_31 = ("1" => "1", "3" => "1", "5" => "1", 
                 "6" => "1", "7" => "1", "10" => "1", "12" => "1");

my %months_30 = ("2" => "1", "4" => "1", "6" => "1", 
                 "9" => "1", "10" => "1");

my %months_28 = ("2" => "1");


# -----------------------------------------------------------------------------
# -- PRINT ROUTINES FOR SEARCH OUTPUT :
# -----------------------------------------------------------------------------

sub results_message() {

    my $str    = shift;
    my $string = <<EEOOFF;
<P>
  <CENTER>
    <FONT FACE="helvetica" SIZE=5><B>
      $str
    </B></FONT>
  </CENTER>
</P>
<P>
  <I>All matching entries in the database are listed --- 
     but you may not have permission to read (or write to) them.
  </I>
</P>
EEOOFF

    return $string;
  }

# -----------------------------------------------------------------------------

sub results_links {

  my $TREE     = shift;
  my $USERNAME = shift;
  my $REALNAME = shift;
  my $results  = shift;

    my $string = "";

    $string .= "<P><BIG><B><FONT FACE=\"helvetica\">" 
            .  "Matching Branches/Entries:" 
            .  "</FONT></B></BIG>\n<BR><TT>";
    $string .= "<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=3>";

    foreach (@$results) {

        my $file = $_->{FILE};

        $file =~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
            # ...hex-encode urls (e.g., for characters such as '&')
            #    see the 'escape' and 'unescape' routines in CGI.pm
            #    also p76: URL encoding, of Webmaster in a Nutshell.
            #    for completeness, note:
            #        $thingy_name =~ s/%([0-9a-fA-F]{2})/pack("c",hex($1))/ge;
            #    decodes...

        $string .= "\n<TR><TD WIDTH=20></TD>";
        $string .= "\n<TD><B><A HREF=\"navigate.pl?" 
                .  "NAVIGATE=NAVIGATE&"
                .  "LOCATION=$_->{LOCATION}&TREE=$TREE&"
                .  "USERNAME=$USERNAME&REALNAME=$REALNAME&" 
                .  "VIEW=ONEONLY&"
                .  "VIEWONEONLY=$_->{FILE}\">$_->{LOCATION}$_->{FILE}</A>"
                .  "</B></TD> ";


        $string .= "<TD> </TD>";
        $string .= "<TD> </TD>";
        if (defined $_->{THINGY}) {
            $string .= "<TD>" . &date_time($_->{THINGY}->time) . "</TD>";
            $string .= "<TD></TD>";
            $string .= "<TD>" . $_->{THINGY}->author . "</TD>";
          }
        $string .= "<TD> </TD>";
        $string .= "<TD> </TD>";
 
        if (defined(@{$_->{KEYWMATCHES}}) >= 1) {
            $string .= "<TD>(kwd matches: <B>";
	    my $k;
            foreach $k (@{$_->{KEYWMATCHES}}) {
                $string .= "$k, ";
              }
            $string .= "</B>)</TD>";
          }

        $string .= "</TR>";
      }
    $string .= "</TABLE>";
    $string .= "</TT></P>\n";

    return $string;
  }


# -----------------------------------------------------------------------------
# -- FIELD VALIDATORS :
# -----------------------------------------------------------------------------

sub constraints_used() {

  my $DATELOWER = shift;
  my $DATEUPPER = shift;
  my $authors   = shift;

    my $string = <<EEOOFF;

<P>
  <B><FONT FACE="helvetica">
    Searching on date constraints:
  </FONT></B>
  <TT>
  <BR>lower: $DATELOWER
  <BR>upper: $DATEUPPER
  </TT>
</P>
<P>
  <B><FONT FACE="helvetica">
    Searching on authors:
  </FONT></B>
  <TT>
EEOOFF

    if (@$authors) {
        foreach (@$authors) {$string .= "\n<BR>$_ ";}
      }
    else {
        $string .= "\n<BR>[No authors]";
      }
    $string .= "</TT></P>\n";

    return $string;
  }

# -----------------------------------------------------------------------------

sub check_author_fields() {

  my $doc     = shift;
  my $except  = shift;
  my $authors = shift;

    foreach (@$authors) {
        if ($_ =~ m/[^a-zA-Z\~]/) {
            $except->raise_stop($doc, "Author field invalid!")
          }
      }
  }

# -----------------------------------------------------------------------------

sub check_date_fields() {

  my $doc    = shift;
  my $except = shift;
  my $lower  = shift;
  my $upper  = shift;

    unless ($lower =~ m/\S/) {$lower = "01/01/1970"}
    unless ($upper =~ m/\S/) {$upper = "31/12/2037"}


    unless ($lower =~ m/^\d\d\/\d\d\/\d\d\d\d$/) {
        $except->raise_stop($doc, "Date fields invalid!")
      }
   
    unless (&date_fields_numerically_valid($lower)) {
        $except->raise_stop($doc, "Date field values (lower: $lower) invalid!")
      }


    unless ($upper =~ m/^\d\d\/\d\d\/\d\d\d\d$/) {
        $except->raise_stop($doc, "Date fields invalid!")
      }

    unless (&date_fields_numerically_valid($upper)) {
        $except->raise_stop($doc, "Date field values (upper) invalid!")
      }

    return ($lower, $upper)
  }


# -----------------------------------------------------------------------------
# --

sub date_fields_numerically_valid() {

  my $string = shift;

    my @date = split(/\//, $string);

    # -- year :

    unless ($date[2] >= 1970) {return 0}
    unless ($date[2] <= 2037) {return 0}


    # -- month :
    
    unless ($date[1] > 0)  {return 0}
    unless ($date[1] < 13) {return 0}


    # -- day :

    $date[1] =~ s/^[0]*//;

    if (defined $months_31{$date[1]}) {
        unless (($date[0] >= 1) && ($date[0] <= 31)) {return 0}
      }

    if (defined $months_30{$date[1]}) {
        unless (($date[0] >= 1) && ($date[0] <= 30)) {return 0}
      }

    if (defined $months_28{$date[1]}) {
        unless (($date[0] >= 1) && ($date[0] <= 28)) {return 0}
      }

    return 1;
  }


# -----------------------------------------------------------------------------
# -- SEARCH CONSTRAINT ONLY :
# -----------------------------------------------------------------------------

# --
#
#

sub constraint_only_search() {
    
  my $doc    = shift;
  my $except = shift;
  my $tree   = shift;

  my $REALNAME = shift;
  my $USERNAME = shift;
  my $TREE     = shift;

  my $DATELOWER = shift;
  my $DATEUPPER = shift;
  my $AUTHORS   = shift;

    $doc->append(&results_message("Constraint-only Search Results"));
    $doc->append(&constraints_used($DATELOWER, $DATEUPPER, $AUTHORS));
    
    # -- search by author, then filter by time :

    my $results;

    $results = $tree->existing_thingies($except);
    $results = &author_filter($results, $AUTHORS);
    $results = &date_filter($results, $DATELOWER, $DATEUPPER);

    $doc->append(&results_links($TREE, $USERNAME, $REALNAME, $results));
  }


# -----------------------------------------------------------------------------
# -- SEARCH KEYWORDS :
# -----------------------------------------------------------------------------

# -- Returns a pointer to an array/list of results, each result being a 
#    hash with some interesting fields in it, including a pointer to 
#    the associated THINGY :

sub keyword_search () {

  my $doc    = shift;
  my $except = shift;
  my $query  = shift;
  my $tree   = shift;

  my $REALNAME = shift;
  my $USERNAME = shift;
  my $TREE     = shift;

  my $DATELOWER = shift;
  my $DATEUPPER = shift;
  my $authors   = shift;

    my $KEYWORDS     = $query->param('KEYWORDS');
    my $KEYWD_BOOLLY = $query->param('KEYWD_BOOLLY');
    my @keywords = split(/\s+/, $KEYWORDS);

    my $results = $tree->existing_thingies($except);
    $results    = &author_filter($results, $authors);
    $results    = &date_filter($results, $DATELOWER, $DATEUPPER);
    
    my @filtered_results = {};

    foreach (@$results) {

        my $kws     = $_->{THINGY}->keywords;
        my @matches = ();
        my $k;

        if ($KEYWD_BOOLLY eq "OR") {
                    foreach $k (@keywords) {
                        foreach (@$kws) {
                            if ($_ eq $k) {push (@matches, $k)}
                          }
                      }
          }
        elsif ($KEYWD_BOOLLY eq "AND") {
            my $and_match = 0;
            foreach $k (@keywords) {
                foreach (@$kws) {
                    if ($_ eq $k) {$and_match += 1;}
                  }
              }
            if ($and_match eq @keywords) {push (@matches, "AND")}
          }
        else {
            $except->raise_logout("Not AND or OR...: /$KEYWD_BOOLLY/");
          }
               
        if (scalar(@matches) >= 1) {
            $_->{KEYWMATCHES} = \@matches;
            push(@filtered_results, $_);
          }           
      }
 
    $doc->append(&results_message("Keyword Search Results: $KEYWD_BOOLLY"));
    $doc->append("<P><B>" .
                 "<FONT FACE=\"helvetica\">Searching on keywords:</FONT>" .
                 "</B><TT>");
    foreach (@keywords) {$doc->append("\n<BR>$_ ");}
    $doc->append("</TT>\n</P>");
    $doc->append(&constraints_used($DATELOWER, $DATEUPPER, $authors));
    $doc->append(&results_links($TREE, $USERNAME, $REALNAME, 
                                \@filtered_results));
  }


# -----------------------------------------------------------------------------
# -- SEARCH GREP : 
# -----------------------------------------------------------------------------


sub grep_entry_names {

  my $doc    = shift;
  my $except = shift;
  my $query  = shift;
  my $tree   = shift;

  my $REALNAME = shift;
  my $USERNAME = shift;
  my $TREE     = shift;

  my $DATELOWER = shift;
  my $DATEUPPER = shift;
  my $authors   = shift;

    my $ENTRIESGREP = $query->param('ENTRIESGREP');
    my $results;

    $results = $tree->existing_thingies($except);
    $results = &author_filter($results, $authors);
    $results = &date_filter($results, $DATELOWER, $DATEUPPER);
    $results = &grep_entry_filter($results, $ENTRIESGREP);

    $doc->append(&results_message("Grep Entry Names Search Results"));

    $doc->append("\n<P><B><FONT FACE=\"helvetica\">Regular-expression:" . 
                 "</FONT></B>");
    $doc->append("\n<BR><TT>$ENTRIESGREP</TT>");

    $doc->append(&constraints_used($DATELOWER, $DATEUPPER, $authors));
    $doc->append(&results_links($TREE, $USERNAME, $REALNAME, $results));
  }


# -----------------------------------------------------------------------------


sub grep_file_names {

  my $doc    = shift;
  my $except = shift;
  my $query  = shift;
  my $tree   = shift;

  my $REALNAME = shift;
  my $USERNAME = shift;
  my $TREE     = shift;

  my $DATELOWER = shift;
  my $DATEUPPER = shift;
  my $authors   = shift;

    my $FILENAMESGREP = $query->param('FILENAMESGREP');
    my $results;

    $results = $tree->existing_thingies($except);
    $results = &author_filter($results, $authors);
    $results = &date_filter($results, $DATELOWER, $DATEUPPER);
    $results = &grep_filenames_filter($results, $FILENAMESGREP);

    $doc->append(&results_message("Grep File Names Search Results"));

    $doc->append("\n<P><B><FONT FACE=\"helvetica\">Regular-expression:" . 
                 "</FONT></B>");
    $doc->append("\n<BR><TT>$FILENAMESGREP</TT>");

    $doc->append(&constraints_used($DATELOWER, $DATEUPPER, $authors));
    $doc->append(&results_links($TREE, $USERNAME, $REALNAME, $results));
  }


# -----------------------------------------------------------------------------
# -- SEARCH SWISH : 
# -----------------------------------------------------------------------------


sub swish_branch_readmes() {

  my $doc      = shift;
  my $except   = shift;
  my $REALNAME = shift;
  my $USERNAME = shift;
  my $TREE     = shift;

  my $root = shift;

  my $SRCH_STR_README = shift;

    my $sw_results = `/usr/local/bin/swish-e -w "$SRCH_STR_README" -f /var/local/doc_U_base/$TREE/SWISH-branch-readmes.index`;

    $doc->append("<P><CENTER><FONT FACE=\"helvetica\" SIZE=7><B>" 
               . "SWISH Branch READMEs:  Results (Tree: $TREE)" 
               . "</B></FONT></CENTER></P>\n");

    $doc->append("\n<P><B><FONT FACE=\"helvetica\">Search-expression:" . 
                 "</FONT></B>");
    $doc->append("\n<BR><TT>$SRCH_STR_README</TT>");

    my $results = &swish_res_to_branches($except, $TREE, $sw_results);
    $doc->append(&results_links($TREE, $USERNAME, $REALNAME, $results));
  }


# -----------------------------------------------------------------------------


sub swish_entry_readmes() {

  my $except   = shift;
  my $REALNAME = shift;
  my $USERNAME = shift;
  my $TREE     = shift;

  my $root = shift;

  my $SRCH_STR_README = shift;

  my $DATELOWER = shift;
  my $DATEUPPER = shift;
  my $authors   = shift;

    my $sw_results = `/usr/local/bin/swish-e -w "$SRCH_STR_README" -f /var/local/doc_U_base/$TREE/SWISH-entry-readmes.index`;

    print "<P><CENTER><FONT FACE=\"helvetica\" SIZE=7><B>" 
      . "SWISH Entry READMEs:  Results (Tree: $TREE)" 
      . "</B></FONT></CENTER></P>\n";

    print "\n<P><B><FONT FACE=\"helvetica\">Search-expression:" . 
          "</FONT></B>";
    print "\n<BR><TT>$SRCH_STR_README</TT>";

    my $results = &swish_res_to_thingies($except, $TREE, $sw_results);

    $results = &author_filter($results, $authors);
    $results = &date_filter($results, $DATELOWER, $DATEUPPER);

    &print_constraints_used($DATELOWER, $DATEUPPER, $authors);
    &print_results_links($TREE, $USERNAME, $REALNAME, $results);
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
