#!/usr/bin/perl -w -I ./_search_lib:./_sal:./_cgi_lib:./_sal_lib:./_Doc:./_config

use strict;

# -----------------------------------------------------------------------------
# History :
# 
#  -- developed early summer 2001;
#  -- first decent version finished 2001 August 03
# 
# -----------------------------------------------------------------------------

use CGI;
use CGI::Carp qw(fatalsToBrowser);

use Cookie;
use Debug;
use Doc;
use Exception;
use Tree;

# -----------------------------------------------------------------------------
use config;
use log;
use sea_mod;
use sea_mod_form;
use thingies;

# -----------------------------------------------------------------------------
# -- MODULE/PACKAGE VARS :

my $cgi_root_dir = &cgi_root_dir;
my $root         = &root;


# -----------------------------------------------------------------------------
# -- INITIALISE :

my $doc     = Doc->new();
my $cookies = Cookie->new($doc);
my $except  = Exception->new();
my $query   = new CGI;
    # ...get pointer to CGI ENV, then extract stuff :

my $TREE     = $query->param('TREE');
my $LOCATION = $query->param('LOCATION');
my $USERNAME = $query->param('USERNAME');
my $REALNAME = $query->param('REALNAME');

my $ACTION      = $query->param('ACTION');
    # ...putting up the menu, or doing a search?

my $SEARCH_TYPE = $query->param('TYPE');

unless ($SEARCH_TYPE) {
    $SEARCH_TYPE = "PRINT_FORM";
  }

# -----------------------------------------------------------------------------
# -- DEBUG :

my $debug = Debug->new(1, "Important ENV variables");

$debug->item("tree:     $TREE");
$debug->item("location: $LOCATION");
$debug->item("username: $USERNAME");
$debug->item("realname: $REALNAME");


# -----------------------------------------------------------------------------
# -- BEGIN OUTPUT :

$doc->html_hdr("", "DocULib Search Widget", "");
$doc->doc_hdr("headersimple", "$USERNAME", $TREE, $LOCATION, "???");
$doc->flush();


# -----------------------------------------------------------------------------
# -- AUTHENTICATE :
# -----------------------------------------------------------------------------

# -- we should have a cookie which should belong to a particular user
#    and if we don't they can bog-off :

unless ($cookies->valid_cookie($doc, $except, $USERNAME)) {
    $except->raise_logout($cookies, "invalid_cookie");
  }


# -----------------------------------------------------------------------------
# -- IF ASKED, PRINT SEARCH FORM AND EXIT :
# -----------------------------------------------------------------------------

if ($ACTION eq 'SEARCH_FORM') {

    $doc->append(&search_form($except, $query, 
                              $USERNAME, $REALNAME, $TREE, $LOCATION,
                              $SEARCH_TYPE));
    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- OK, ASSUME WE'RE ACTUALLY DOING A SEARCH (THE PARTICULAR SEARCH WILL BE
#    DETERMINED BY THE BUTTON CLICKED!) :
# -----------------------------------------------------------------------------

# -- first, get further ENV stuff, then check fields ok, then extract 
#    authors-list from AUTHORS : 

my $BUTTON = $query->param('BUTTON');

my $AUTHORS   = $query->param('AUTHORS');
my $DATELOWER = $query->param('DATELOWER');
my $DATEUPPER = $query->param('DATEUPPER');

($DATELOWER, $DATEUPPER) 
                  = &check_date_fields($doc, $except, $DATELOWER, $DATEUPPER);

unless ($AUTHORS) {$AUTHORS = ""}
my @authors = split(/\s+/, $AUTHORS);
&check_author_fields($doc, $except, \@authors);


# -----------------------------------------------------------------------------
# -- OUR OBJECTS :

my $tree = Tree->new($except, $TREE);


# -----------------------------------------------------------------------------
# -- DO A SEARCH DEPENDING ON THE BUTTON :

if ($BUTTON eq 'Constraint-only Search') {

    &constraint_only_search($doc, $except, $tree,
                            $REALNAME, $USERNAME, $TREE, 
                            $DATELOWER, $DATEUPPER, \@authors);
  }

# -----------------------------------------------------------------------------

if ($BUTTON eq 'Keyword Search') {

    &keyword_search($doc, $except, $query, $tree, 
                    $REALNAME, $USERNAME, $TREE, 
                    $DATELOWER, $DATEUPPER, \@authors);
  }

# -----------------------------------------------------------------------------

if ($BUTTON eq 'Grep Entries') {
    &grep_entry_names($doc, $except, $query, $tree,
                      $REALNAME, $USERNAME, $TREE, 
                      $DATELOWER, $DATEUPPER, \@authors);
  }

# -----------------------------------------------------------------------------

if ($BUTTON eq 'Grep Filenames') {
    &grep_file_names($doc, $except, $query, $tree,
                     $REALNAME, $USERNAME, $TREE, 
                     $DATELOWER, $DATEUPPER, \@authors);
  }

# -----------------------------------------------------------------------------

if ($BUTTON eq 'SWISH Branch READMEs') {

    my $SRCH_STR_README_Branch = $query->param('SRCH_STR_README_Branch');

    &swish_branch_readmes($doc, $except,
                          $REALNAME, $USERNAME, $TREE, 
                          $root,
                          $SRCH_STR_README_Branch);
  }

# -----------------------------------------------------------------------------

if ($BUTTON eq 'SWISH Entry READMEs') {

    my $SRCH_STR_README_Entry = $query->param('SRCH_STR_README_Entry');

    &swish_entry_readmes($doc, $except,
                         $REALNAME, $USERNAME, $TREE, 
                         $root,
                         $SRCH_STR_README_Entry,
                         $DATELOWER, $DATEUPPER, \@authors);
  }

# -----------------------------------------------------------------------------
$doc->wen();
# -----------------------------------------------------------------------------

