#!/usr/bin/perl -w -I ./_cgi_lib -I _Doc -I _sal -I ./_config -I ./_sal_lib 

use strict;

# -----------------------------------------------------------------------------
# -- CGI :

use CGI;
use CGI::Carp qw(fatalsToBrowser);

# -----------------------------------------------------------------------------
# -- LIB :

use config;
use log;
use up_mod;

# -----------------------------------------------------------------------------
# -- SAL :

use CDF;
use Cookie;
use Doc;
use Exception;
use State;
use Thingy;

# -----------------------------------------------------------------------------
# -- INITIALISATION :

my $doc     = Doc->new();
my $cookies = Cookie->new($doc);
my $except  = Exception->new();
my $html    = CGI->new();
my $s       = State->new($cookies, $doc, $except);

my $upfile = {};
    # ...details of the file(s) we are gonna upload;
   

# -----------------------------------------------------------------------------
# -- DOC(HTML) BEGIN :

$doc->html_hdr("", "Contents", "");


# -----------------------------------------------------------------------------
# -- given where we are in the tree, "get" the CDF and who our parent is : 

my $cdf    = CDF->new_2($cookies, $doc, $except, $s);
my $parent = $cdf->parent;  


# -- get stuff from the installation configuration :

my $root         = &root;
my $cgi_root_dir = &cgi_root_dir;
my $tmp_dir      = &tmp_dir;


# -----------------------------------------------------------------------------
# -- This helps "show progress" a bit on a big/slow upload...

$| = 1;

# -----------------------------------------------------------------------------
# -- DOC(HEADER) BEGIN :

$doc->doc_hdr_2("headerstd", $s, $parent);


$doc->append("ACTION: " . $s->attr("ACTION"));

# -----------------------------------------------------------------------------
# -- AUTHENTICATE :

unless ($cookies->valid_cookie($doc, $except, $s->attr("USERNAME"))) {
    $except->raise_logout($cookies, $doc, "invalid_cookie")
  }


# -----------------------------------------------------------------------------
# -- First, are we cancelled?

if (($s->def("BUTTON")) && ($s->attr("BUTTON") eq 'Cancel')) {
    $doc->append("<H1>Cancelled!</H1>");
    $doc->append(&refresh_to_navigate($s->attr("TREE"), $s->attr("LOCATION"), 
                                      $s->attr("USERNAME"), 
                                      $s->attr("REALNAME")));
    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- The upload-new-document-form :

if ($s->attr("ACTION") eq 'FORM_NEW_DOC') {
    unless ($cdf->write_permission_granted($except, $s->attr("USERNAME"), 
                                           $s->attr("TREE"), 
                                           $s->attr("LOCATION"))) {
        $except->raise_options($doc, $s,
                               "You don't have permission " . 
                               "to write to this tree/location.");
        $doc->wen();
        exit;
      }
    $doc->append(&upload_form($html, $s));
    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- the upload new document form :

if ($s->attr("ACTION") eq 'FORM_NEW_VERSION') {

    unless ($cdf->write_permission_granted($except, $s->attr("USERNAME"), 
                                           $s->attr("TREE"), 
                                           $s->attr("LOCATION"))) {
        $except->raise_options($doc, $s,
                               "You don't have permission " . 
                               "to write to this tree/location.");
        $doc->wen();
        exit;
      }
    $doc->append(&upload_nv_form($html, $s));
    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- IF WE GOT THIS FAR WE'RE GOING FOR AN UPLOAD OF SOME KIND :
# -----------------------------------------------------------------------------

# -- first, check permissions :

unless ($cdf->write_permission_granted($except, $s->attr("USERNAME"), 
                                                $s->attr("TREE"), 
                                                $s->attr("LOCATION"))) {
    $except->raise_options($doc, $s,
                           "You don't have permission " . 
                           "to write to this tree/location.");
  }


# -----------------------------------------------------------------------------
# -- get upload stuff from the environment and, perhaps, debug it :

my $debug = Debug->new(1, "Important ENV variables");

$debug->item("upload.pl");
$debug->item("def file : " . $s->attr("UP_FILE_DEF"));
$debug->item("view file: " . $s->attr("UP_FILE_VIEW"));
$debug->item("Keywords : " . $s->attr("KEYWORDS"));
$debug->item("Readme   : " . $s->attr("README"));
$debug->item("Action   : " . $s->attr("ACTION"));


# -----------------------------------------------------------------------------
# -- strip any path from the file (guilty: IE);  eliminate spaces : 

$s->{DOCNAMEROOT} = &clean_doc_root($s->{DOCNAMEROOT});

my $pf_up_file_def  = &clean_file_name($s->attr("UP_FILE_DEF"));
my $pf_up_file_view = &clean_file_name($s->attr("UP_FILE_VIEW"));

$debug->item("pf def file: $pf_up_file_def");
$debug->item("pf view file: $pf_up_file_view");
$debug->wen($doc);


# -----------------------------------------------------------------------------
# -- CASE OF CHECK UPLOAD OF NEW VER :
# -----------------------------------------------------------------------------

if ($s->attr("ACTION") eq 'UPLOAD_NEW_VERSION') {

    &check_ok_to_move_nv_from_tmp();

    &move_from_tmp($doc, $except, $s->attr("TREE"), $s->attr("LOCATION"), 
                   $s->{DOCNAMEROOT}, $pf_up_file_def, "NEW_VER");
    &move_from_tmp($doc, $except, $s->attr("TREE"), $s->attr("LOCATION"), 
                   $s->attr("DOCNAMEROOT"), $pf_up_file_view, "NEW_VER");

    &build_and_write_nv_thingy($doc, $except, $s->attr("TREE"), 
                               $s->attr("LOCATION"),
                               $s->attr("KEYWORDS"), $s->attr("AUTHOR"), 
                               $s->attr("REALNAME"), $s->attr("DATEWRITTEN"), 
                               $s->attr("HISTORYAPPEND"), 
                               $s->attr("DOCNAMEROOT"), 
                               $pf_up_file_def, $pf_up_file_view);

    $doc->append(&refresh_to_navigate($s->attr("TREE"), $s->attr("LOCATION"), 
                                      $s->attr("USERNAME"), 
                                      $s->attr("REALNAME}")));

#    log it ...............

    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- CASE OF UPLOAD NEW VER OF EXISTING ENTRY/THINGY TO TMP :
# -----------------------------------------------------------------------------

if ($s->attr("ACTION") eq 'CHECK_UPLOAD_NEW_VERSION') {

    &check_constraints_on_nv($doc, $except, $s, 
                             $pf_up_file_def, $pf_up_file_view);

    &log_upload_new_ver($s->{USERNAME}, $pf_up_file_def, $pf_up_file_view);

    $doc->append(&upload_file($doc, $except, 
                     $s->attr("UP_FILE_DEF"), $pf_up_file_def, 
                     "$tmp_dir" . $s->attr("DOCNAMEROOT") . "--$pf_up_file_def"));

    $doc->append(&upload_file($doc, $except,
                     $s->attr("UP_FILE_VIEW"), $pf_up_file_view, 
                     "$tmp_dir" . $s->attr("DOCNAMEROOT") . "--$pf_up_file_view"));

    $doc->append(&really_do_this($doc, $except, 
                $s->attr("REALNAME"), $s->attr("USERNAME"), $s->attr("LOCATION"), $s->attr("TREE"), $s->attr("AUTHOR"), $s->attr("DATEWRITTEN"), 
                $pf_up_file_def, $pf_up_file_view,
                $s->attr("KEYWORDS"), $s->attr("README"), $s->attr("DOCNAMEROOT"),
                "UPLOAD_NEW_VERSION"));

    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- CASE OF UPLOAD TO TMP, ASK USER REALLY DO THIS? 
# -----------------------------------------------------------------------------


if ($s->attr("ACTION") eq 'CHECK_UPLOAD') {
 
    $doc->append(&upload_file($doc, $except,
                     $s->attr("UP_FILE_DEF"), $pf_up_file_def, 
                     "$tmp_dir" . $s->attr("DOCNAMEROOT") . "--$pf_up_file_def"));

    $doc->append(&upload_file($doc, $except, 
                     $s->attr("UP_FILE_VIEW"), $pf_up_file_view, 
                     "$tmp_dir" . $s->attr("DOCNAMEROOT") . "--$pf_up_file_view"));

    $doc->append(&really_do_this($doc, $except, 
                $s->attr("REALNAME"), $s->attr("USERNAME"), $s->attr("LOCATION"), $s->attr("TREE"), $s->attr("AUTHOR"), $s->attr("DATEWRITTEN"), 
                $pf_up_file_def, $pf_up_file_view,
                $s->attr("KEYWORDS"), $s->attr("README"), $s->attr("DOCNAMEROOT"), "UPLOAD_NEW_DOC"));

    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- CASE OF UPLOADING NEW ENTRY/THINGY (MOVING FROM TMP TO DATABASE) :
# -----------------------------------------------------------------------------

if ($s->attr("ACTION") eq 'UPLOAD_NEW_DOC') {

    &check_ok_to_move_from_tmp($doc, $except, $s, 
                               $pf_up_file_def, $pf_up_file_view);

    &move_from_tmp($doc, $except, $s->{TREE}, $s->{LOCATION},
                   $s->attr("DOCNAMEROOT"), $pf_up_file_def);
    &move_from_tmp($doc, $except, $s->attr("TREE"), $s->attr("LOCATION"),
                   $s->attr("DOCNAMEROOT"), $pf_up_file_view);

    &build_and_write_thingy($doc, $except, 
              $s->attr("TREE"), $s->attr("LOCATION"),
              $s->attr("KEYWORDS"), $s->attr("AUTHOR"), $s->attr("REALNAME"), 
              $s->attr("DATEWRITTEN"), $s->attr("README"), 
              $s->attr("DOCNAMEROOT"), $pf_up_file_def, $pf_up_file_view);


    &log_upload_new_doc($s->attr("USERNAME"), $pf_up_file_def, $pf_up_file_view);

    $doc->append(&refresh_to_navigate($s->attr("TREE"), $s->attr("LOCATION"), 
                                      $s->attr("USERNAME"), $s->attr("REALNAME")));
    $doc->wen();
    exit;
  }


# -----------------------------------------------------------------------------
# -- IF WE'VE GOT THIS FAR WE HAVE AN ERROR :
# -----------------------------------------------------------------------------

$doc->append("It's all gone horribly wrong!");

$doc->wen();
exit;


# -----------------------------------------------------------------------------

